@@ -41,7 +41,6 @@
 #include <unistd.h>
 #include <glib/gi18n-lib.h>
 
-#include "vteunistr.h"
 #include "vte.h"
 #include "buffer.h"
 #include "debug.h"
@@ -53,7 +52,6 @@
 
 G_BEGIN_DECLS
 
-#define VTE_PAD_WIDTH			1
 #define VTE_TAB_WIDTH			8
 #define VTE_LINE_WIDTH			1
 #define VTE_ROWS			24
@@ -62,15 +60,10 @@ G_BEGIN_DECLS
 #define VTE_COLOR_PLAIN_OFFSET		0
 #define VTE_COLOR_BRIGHT_OFFSET		8
 #define VTE_COLOR_DIM_OFFSET		16
-#define VTE_DEF_FG			256
-#define VTE_DEF_BG			257
-#define VTE_BOLD_FG			258
-#define VTE_DIM_FG			259
-#define VTE_DEF_HL                      260
-#define VTE_CUR_BG			261
+/* More color defines in ring.h */
 
-#define VTE_SATURATION_MAX		10000
 #define VTE_SCROLLBACK_INIT		100
+#define VTE_SATURATION_MAX		10000
 #define VTE_DEFAULT_CURSOR		GDK_XTERM
 #define VTE_MOUSING_CURSOR		GDK_LEFT_PTR
 #define VTE_TAB_MAX			999
@@ -96,39 +89,6 @@ G_BEGIN_DECLS
 #define I_(string) (g_intern_static_string(string))
 
 
-/* The structure we use to hold characters we're supposed to display -- this
- * includes any supported visible attributes. */
-struct vte_charcell {
-	vteunistr c;		/* The Unicode string for the cell. */
-
-	struct vte_charcell_attr {
-		guint32 columns: 4;	/* Number of visible columns
-					   (as determined by g_unicode_iswide(c)).
-					   Also abused for tabs; bug 353610
-					   Keep at least 4 for tabs to work
-					   */
-		guint32 fore: 9;	/* Index into color palette */
-		guint32 back: 9;	/* Index into color palette. */
-
-		guint32 fragment: 1;	/* A continuation cell. */
-		guint32 standout: 1;	/* Single-bit attributes. */
-		guint32 underline: 1;
-		guint32 strikethrough: 1;
-
-		guint32 reverse: 1;
-		guint32 blink: 1;
-		guint32 half: 1;
-		guint32 bold: 1;
-
-		guint32 invisible: 1;
-		/* unused; bug 499893
-		guint32 protect: 1;
-		 */
-
-		/* 31 bits */
-	} attr;
-};
-
 typedef enum {
         VTE_REGEX_GREGEX,
         VTE_REGEX_VTE,
@@ -183,17 +143,15 @@ typedef struct _VteWordCharRange {
 	gunichar start, end;
 } VteWordCharRange;
 
-typedef struct _VteRowData {
-	GArray *cells;
-	guchar soft_wrapped: 1;
-} VteRowData;
+typedef struct _VteVisualPosition {
+	long row, col;
+} VteVisualPosition;
 
 /* Terminal private data. */
 struct _VteTerminalPrivate {
 	/* Emulation setup data. */
 	struct _vte_termcap *termcap;	/* termcap storage */
 	struct _vte_matcher *matcher;	/* control sequence matcher */
-	const char *termcap_path;	/* path to termcap file */
 	const char *emulation;		/* terminal type to emulate */
 	struct vte_terminal_flags {	/* boolean termcap flags */
 		gboolean am;
@@ -214,8 +172,7 @@ struct _VteTerminalPrivate {
 	int default_column_count, default_row_count;	/* default sizes */
 
 	/* PTY handling data. */
-	const char *shell;		/* shell we started */
-	int pty_master;			/* pty master descriptor */
+	VtePty *pty;
 	GIOChannel *pty_channel;	/* master channel */
 	guint pty_input_source;
 	guint pty_output_source;
@@ -241,20 +198,17 @@ struct _VteTerminalPrivate {
 	glong max_input_bytes;
 
 	/* Output data queue. */
-	struct _vte_buffer *outgoing;	/* pending input characters */
+	VteBuffer *outgoing;	/* pending input characters */
 	VteConv outgoing_conv;
 
 	/* IConv buffer. */
-	struct _vte_buffer *conv_buffer;
+	VteBuffer *conv_buffer;
 
 	/* Screen data.  We support the normal screen, and an alternate
 	 * screen, which seems to be a DEC-specific feature. */
 	struct _VteScreen {
-		VteRing *row_data;	/* row data, arranged as a GArray of
-					   vte_charcell structures */
-		struct vte_cursor_position {
-			long row, col;
-		} cursor_current, cursor_saved;
+		VteRing row_data[1];	/* buffer contents */
+		VteVisualPosition cursor_current, cursor_saved;
 					/* the current and saved positions of
 					   the [insertion] cursor -- current is
 					   absolute, saved is relative to the
@@ -264,34 +218,34 @@ struct _VteTerminalPrivate {
 		gboolean sendrecv_mode;	/* sendrecv mode */
 		gboolean insert_mode;	/* insert mode */
 		gboolean linefeed_mode;	/* linefeed mode */
+		gboolean bracketed_paste_mode;
 		struct vte_scrolling_region {
 			int start, end;
 		} scrolling_region;	/* the region we scroll in */
 		gboolean scrolling_restricted;
 		long scroll_delta;	/* scroll offset */
 		long insert_delta;	/* insertion offset */
-		struct vte_charcell defaults;	/* default characteristics
-						   for insertion of any new
-						   characters */
-		struct vte_charcell color_defaults;	/* original defaults
-							   plus the current
-							   fore/back */
-		struct vte_charcell fill_defaults;	/* original defaults
-							   plus the current
-							   fore/back with no
-							   character data */
-		struct vte_charcell basic_defaults;	/* original defaults */
+		VteCell defaults;	/* default characteristics
+					   for insertion of any new
+					   characters */
+		VteCell color_defaults;	/* original defaults
+					   plus the current
+					   fore/back */
+		VteCell fill_defaults;	/* original defaults
+					   plus the current
+					   fore/back with no
+					   character data */
 		gboolean alternate_charset;
 		gboolean status_line;
 		GString *status_line_contents;
 		gboolean status_line_changed;
 	} normal_screen, alternate_screen, *screen;
-	VteRowData *free_row;
 
 	/* Selection information. */
 	GArray *word_chars;
 	gboolean has_selection;
 	gboolean selecting;
+	gboolean selecting_after_threshold;
 	gboolean selecting_restart;
 	gboolean selecting_had_delta;
 	gboolean selection_block_mode;
@@ -304,9 +258,7 @@ struct _VteTerminalPrivate {
 	struct selection_event_coords {
 		long x, y;
 	} selection_origin, selection_last;
-	struct selection_cell_coords {
-		long row, col;
-	} selection_start, selection_end;
+	VteVisualPosition selection_start, selection_end;
 
 	/* Miscellaneous options. */
 	VteTerminalEraseBinding backspace_binding, delete_binding;
@@ -331,6 +283,7 @@ struct _VteTerminalPrivate {
 
 	/* Cursor shape */
 	VteTerminalCursorShape cursor_shape;
+        float cursor_aspect_ratio;
 
 	/* Cursor blinking. */
         VteTerminalCursorBlinkMode cursor_blink_mode;
@@ -341,6 +294,7 @@ struct _VteTerminalPrivate {
         gboolean cursor_blinks;           /* whether the cursor is actually blinking */
 	gint64 cursor_blink_time;         /* how long the cursor has been blinking yet */
 	gboolean cursor_visible;
+	gboolean has_focus;               /* is the terminal window focused */
 
 	/* Input device options. */
 	time_t last_keypress_time;
@@ -351,6 +305,8 @@ struct _VteTerminalPrivate {
 	long mouse_last_x, mouse_last_y;
 	gboolean mouse_autohide;
 	guint mouse_autoscroll_tag;
+	gboolean mouse_xterm_extension;
+	gboolean mouse_urxvt_extension;
 
 	/* State variables for handling match checks. */
 	char *match_contents;
@@ -359,11 +315,14 @@ struct _VteTerminalPrivate {
 	GArray *match_regexes;
 	char *match;
 	int match_tag;
-	struct {
-		long row, column;
-	} match_start, match_end;
+	VteVisualPosition match_start, match_end;
 	gboolean show_match;
 
+	/* Search data. */
+	GRegex *search_regex;
+	gboolean search_wrap_around;
+	GArray *search_attrs; /* Cache attrs */
+
 	/* Data used when rendering the text which does not require server
 	 * resources and which can be kept after unrealizing. */
 	PangoFontDescription *fontdesc;
@@ -378,9 +337,7 @@ struct _VteTerminalPrivate {
 	gboolean palette_initialized;
 	gboolean highlight_color_set;
 	gboolean cursor_color_set;
-	struct vte_palette_entry {
-		guint16 red, green, blue;
-	} palette[VTE_CUR_BG + 1];
+	PangoColor palette[VTE_PALETTE_SIZE];
 
 	/* Mouse cursors. */
 	gboolean mouse_cursor_visible;
@@ -407,6 +364,10 @@ struct _VteTerminalPrivate {
 	/* window name changes */
 	gchar *window_title_changed;
 	gchar *icon_title_changed;
+        gchar *current_directory_uri;
+        gchar *current_directory_uri_changed;
+        gchar *current_file_uri;
+        gchar *current_file_uri_changed;
 
 	/* Background images/"transparency". */
 	guint root_pixmap_changed_tag;
@@ -414,7 +375,7 @@ struct _VteTerminalPrivate {
 	gboolean bg_transparent;
 	GdkPixbuf *bg_pixbuf;
 	char *bg_file;
-	GdkColor bg_tint_color;
+	PangoColor bg_tint_color;
 	guint16 bg_saturation;	/* out of VTE_SATURATION_MAX */
 	guint16 bg_opacity;
 
@@ -429,8 +390,24 @@ struct _VteTerminalPrivate {
 	glong line_thickness;
 	glong underline_position;
 	glong strikethrough_position;
+
+        /* Style stuff */
+        GtkBorder inner_border;
+
+#if GTK_CHECK_VERSION (2, 91, 2)
+        /* GtkScrollable impl */
+        GtkAdjustment *hadjustment; /* unused */
+        guint hscroll_policy : 1; /* unused */
+
+        guint vscroll_policy : 1;
+#endif
 };
 
+#if GTK_CHECK_VERSION (2, 99, 0)
+struct _VteTerminalClassPrivate {
+        GtkStyleProvider *style_provider;
+};
+#endif
 
 VteRowData *_vte_terminal_ensure_row(VteTerminal *terminal);
 void _vte_terminal_set_pointer_visible(VteTerminal *terminal, gboolean visible);
@@ -441,9 +418,6 @@ void _vte_invalidate_cells(VteTerminal *terminal,
 void _vte_invalidate_cell(VteTerminal *terminal, glong col, glong row);
 void _vte_invalidate_cursor_once(VteTerminal *terminal, gboolean periodic);
 VteRowData * _vte_new_row_data(VteTerminal *terminal);
-VteRowData * _vte_new_row_data_sized(VteTerminal *terminal, gboolean fill);
-VteRowData * _vte_reset_row_data (VteTerminal *terminal, VteRowData *row, gboolean fill);
-void _vte_free_row_data(VteRowData *row);
 void _vte_terminal_adjust_adjustments(VteTerminal *terminal);
 void _vte_terminal_queue_contents_changed(VteTerminal *terminal);
 void _vte_terminal_emit_text_deleted(VteTerminal *terminal);
@@ -466,6 +440,10 @@ void _vte_terminal_beep(VteTerminal *terminal);
 
 void _vte_terminal_inline_error_message(VteTerminal *terminal, const char *format, ...) G_GNUC_PRINTF(2,3);
 
+VteRowData *_vte_terminal_ring_insert (VteTerminal *terminal, glong position, gboolean fill);
+VteRowData *_vte_terminal_ring_append (VteTerminal *terminal, gboolean fill);
+void _vte_terminal_ring_remove (VteTerminal *terminal, glong position);
+
 /* vteseq.c: */
 void _vte_terminal_handle_sequence(VteTerminal *terminal,
 				   const char *match_s,
